@@ -2,21 +2,25 @@
 
 import argparse
 import logging
-import multiprocessing
 import os
 import os.path
 import shutil
 import sys
 import time
 import datetime
+import psutil
+import platform
+import tempfile
 
 import importlib.util
 
+destdir = os.getenv("DESTDIR", "")
+
 # If the cvise modules cannot be found
 # add the known install location to the path
 if importlib.util.find_spec("cvise") is None:
-    sharedir = "@CMAKE_INSTALL_FULL_DATADIR@"
-    sys.path.append(sharedir)
+    sys.path.append("@CMAKE_INSTALL_FULL_DATADIR@")
+    sys.path.append(destdir + "@CMAKE_INSTALL_FULL_DATADIR@")
 
 from cvise import CVise
 from cvise.passes.abstract import AbstractPass
@@ -37,7 +41,8 @@ def get_share_dir():
 
     # Test all known locations for the cvise directory
     share_dirs = [
-            "@CMAKE_INSTALL_FULL_DATADIR@/@cvise_PACKAGE@",
+            os.path.join("@CMAKE_INSTALL_FULL_DATADIR@", "@cvise_PACKAGE@"),
+            destdir + os.path.join("@CMAKE_INSTALL_FULL_DATADIR@", "@cvise_PACKAGE@"),
             os.path.join(script_path, "cvise")
             ]
 
@@ -59,10 +64,21 @@ def find_external_programs():
         path = None
         if local_folder:
             local_folder = os.path.join(script_path, local_folder)
+            if platform.system() == "Windows":
+                for configuration in ["Debug", "Release"]:
+                    new_local_folder = os.path.join(local_folder, configuration)
+                    if os.path.exists(new_local_folder):
+                        local_folder = new_local_folder
+                        break
+
             path = shutil.which(prog, path=local_folder)
 
             if not path:
-                path = shutil.which(prog, path="@CMAKE_INSTALL_FULL_LIBEXECDIR@/@cvise_PACKAGE@/")
+                search = os.path.join("@CMAKE_INSTALL_FULL_LIBEXECDIR@", "@cvise_PACKAGE@")
+                path = shutil.which(prog, path=search)
+            if not path:
+                search = destdir + os.path.join("@CMAKE_INSTALL_FULL_LIBEXECDIR@", "@cvise_PACKAGE@")
+                path = shutil.which(prog, path=search)
 
         if not path:
             path = shutil.which(prog)
@@ -94,7 +110,7 @@ def get_available_pass_groups():
 
         try:
             pass_group_dict = CVise.load_pass_group_file(path)
-            CVise.parse_pass_group_dict(pass_group_dict, set(), None, None, None, None)
+            CVise.parse_pass_group_dict(pass_group_dict, set(), None, None, None, None, None)
         except MissingPassGroupsError:
             logging.warning("Skipping file {}. Not valid pass group.".format(path))
         else:
@@ -114,7 +130,14 @@ For bug reporting instructions, please use:
 
 if __name__ == "__main__":
     try:
-        core_count = multiprocessing.cpu_count()
+        # try to detect only physical cores, ignore HyperThreading
+        # in order to speed up parallel execution
+        core_count = psutil.cpu_count(logical=False)
+        if not core_count:
+            psutil.cpu_count(logical=True)
+        # respect affinity
+        affinity = len(psutil.Process().cpu_affinity())
+        core_count = min(core_count, affinity)
     except NotImplementedError:
         core_count = 1
 
@@ -133,8 +156,8 @@ if __name__ == "__main__":
     parser.add_argument("--save-temps", action="store_true", default=False, help="Don't delete /tmp/cvise-xxxxxx directories on termination")
     parser.add_argument("--skip-initial-passes", action="store_true", default=False, help="Skip initial passes (useful if input is already partially reduced)")
     parser.add_argument("--remove-pass", help="Remove all instances of the specified passes from the schedule (comma-separated)")
-    parser.add_argument("--timing", action="store_true", default=False, help="Print timestamps about reduction progress")
-    parser.add_argument("--timing-since-start", action="store_true", default=False, help="Print timestamps since the start of a reduction")
+    parser.add_argument("--no-timing", action="store_true", default=False, help="Do not print timestamps about reduction progress")
+    parser.add_argument("--timestamp", action="store_true", default=False, help="Print timestamps instead of relative time from a reduction start")
     parser.add_argument("--timeout", type=int, nargs="?", const=300, help="Interestingness test timeout in seconds")
     parser.add_argument("--no-cache", action="store_true", default=False, help="Don't cache behavior of passes")
     parser.add_argument("--skip-key-off", action="store_true", default=False, help="Disable skipping the rest of the current pass when \"s\" is pressed")
@@ -144,9 +167,11 @@ if __name__ == "__main__":
     passes_group.add_argument("--pass-group-file", type=str, help="JSON file defining a custom pass group")
     parser.add_argument("--clang-delta-std", type=str, choices=["c++98", "c++11", "c++14", "c++17", "c++20"], help="Specify clang_delta C++ standard, it can rapidly speed up all clang_delta passes")
     parser.add_argument("--not-c", action="store_true", help="Don't run passes that are specific to C and C++, use this mode for reducing other languages")
+    parser.add_argument("--no-renaming", action="store_true", help="Skip all renaming passes")
     parser.add_argument("--list-passes", action="store_true", help="Print all available passes and exit")
-    parser.add_argument("--version", action="version", version='%s (%s)' % (CVise.Info.PACKAGE_STRING, CVise.Info.GIT_VERSION))
-    parser.add_argument("interestingness_test", metavar="INTERESTINGNESS_TEST", help="Executable to check interestingness of test cases")
+    parser.add_argument("--version", action="version", version=CVise.Info.PACKAGE_STRING  + (' (%s)' % CVise.Info.GIT_VERSION if CVise.Info.GIT_VERSION != 'unknown' else ''))
+    parser.add_argument("--commands", "-c", help="Use bash commands instead of an interestingness test case")
+    parser.add_argument("interestingness_test", metavar="INTERESTINGNESS_TEST", nargs="?", help="Executable to check interestingness of test cases")
     parser.add_argument("test_cases", metavar="TEST_CASE", nargs="+", help="Test cases")
 
     args = parser.parse_args()
@@ -154,11 +179,11 @@ if __name__ == "__main__":
     log_config = {}
 
     log_format = "%(levelname)s %(message)s"
-    if args.timing:
-        if args.timing_since_start:
-            log_format = "%(delta)s " + log_format
-        else:
+    if not args.no_timing:
+        if args.timestamp:
             log_format = "%(asctime)s " + log_format
+        else:
+            log_format = "%(delta)s " + log_format
 
     if args.debug:
         log_config["level"] = logging.DEBUG
@@ -194,7 +219,7 @@ if __name__ == "__main__":
 
     pass_group_dict = CVise.load_pass_group_file(pass_group_file)
     pass_group = CVise.parse_pass_group_dict(pass_group_dict, pass_options, external_programs,
-            args.remove_pass, args.clang_delta_std, args.not_c)
+            args.remove_pass, args.clang_delta_std, args.not_c, args.no_renaming)
     if args.list_passes:
         logging.info('Available passes:')
         logging.info('INITIAL PASSES')
@@ -212,6 +237,19 @@ if __name__ == "__main__":
 
     pass_statistic = statistics.PassStatistic()
 
+    if not args.interestingness_test and not args.commands:
+        print('Either INTERESTINGNESS_TEST or --commands must be used!')
+        exit(1)
+
+    script = None
+    if args.commands:
+        with tempfile.NamedTemporaryFile(mode="w", delete=False, suffix=".sh") as script:
+            script.write('#!/bin/bash\n\n')
+            script.write(args.commands + '\n')
+        os.chmod(script.name, 0o744)
+        logging.info('Using temporary interestingness test: %s' % script.name)
+        args.interestingness_test = script.name
+
     test_manager = testing.TestManager(pass_statistic, args.interestingness_test, args.timeout,
             args.save_temps, args.test_cases, args.n, args.no_cache, args.skip_key_off, args.shaddap,
             args.die_on_pass_bug, args.print_diff, args.max_improvement, args.no_give_up, args.also_interesting)
@@ -240,11 +278,14 @@ if __name__ == "__main__":
                 pass_data.worked, pass_data.failed, pass_data.totally_executed))
         print()
 
+        if not args.no_timing:
+            print("Runtime: {} seconds".format(round((time_stop - time_start))))
+
+        print('Reduced test-cases:\n')
         for test_case in test_manager.sorted_test_cases:
             with open(test_case) as test_case_file:
                 print(test_case_file.read())
-
-    if args.timing:
-        print("Runtime: {} seconds".format(round((time_stop - time_start))))
+        if script:
+            os.remove(script.name)
 
     logging.shutdown()
