@@ -1,7 +1,4 @@
-import filecmp
 import os
-import shutil
-import subprocess
 import tempfile
 import re
 
@@ -48,7 +45,7 @@ class IfPass(AbstractPass):
             state.value = 0
         return state
 
-    def advance_on_success(self, test_case, state, process_event_notifier):
+    def advance_on_success(self, test_case, state):
         return state.advance_on_success(self.__count_instances(test_case))
 
     def transform(self, test_case, state, process_event_notifier):
